@@ -8,7 +8,7 @@ my $VALID_NAME = qr{ ^ [^\W0-9] \w* $ }ix;
 
 package Moos;
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Scalar::Util;
 use Carp qw(confess);
@@ -224,8 +224,29 @@ sub add_attribute {
     );
 }
 
+{
+    my $has_subname = eval { require Sub::Name; 1 };
+    sub add_method {
+        my ($self, $name, $code) = @_;
+        my $pkg = $self->name;
+        if (ref $code) {
+            if ($has_subname) {
+                $code = Sub::Name::subname("$pkg\::$name", $code);
+                Moos::_export($pkg, $name, $code);
+            }
+            else {
+                # close over $code
+                eval "package $pkg; sub $name { goto \$code }";
+            }
+        }
+        else {
+            eval "package $pkg; sub $name { $code }";
+        }
+    }
+}
+
 # A tracing wrapper for debugging accessors
-my $trace_exclude = +{
+our $TRACE_EXCLUDE = +{
     map {($_, 1)} (
         'Some::Module some_accessor',
         'Some::Module some_other_accessor',
@@ -236,7 +257,7 @@ sub _trace_accessor_calls {
     my ($name, $accessor) = @_;
     sub {
         my ($pkg, $file, $line, $sub) = caller(0);
-        unless ($trace_exclude->{"$pkg $name"}) {
+        unless ($TRACE_EXCLUDE->{"$pkg $name"}) {
             warn "$pkg $name $line\n";
             Time::HiRes::usleep(100000);
         }
@@ -511,9 +532,9 @@ sub _setup_accessor
         }
         else {
             my $accessor = $self->{is} eq 'ro'
-                ? eval qq{ sub { Carp::confess("cannot set value for read-only accessor '$name'") if \@_ > 1; \$_[0]{'$name'} } }
-                : eval qq{ sub { \$#_ ? \$_[0]{'$name'} = \$_[1] : \$_[0]{'$name'} } };
-            return Moos::_export($metaclass->{package}, $name, $accessor);
+                ? qq{ Carp::confess("cannot set value for read-only accessor '$name'") if \@_ > 1; \$_[0]{'$name'} }
+                : qq{ \$#_ ? \$_[0]{'$name'} = \$_[1] : \$_[0]{'$name'} };
+            return $metaclass->add_method($name, $accessor);
         }
     }
 
@@ -560,7 +581,7 @@ sub _setup_accessor
         if $ENV{PERL_MOOS_ACCESSOR_CALLS};
 
     # Export the accessor.
-    Moos::_export($metaclass->{package}, $name, $accessor);
+    $metaclass->add_method($name, $accessor);
 
     return;
 }
@@ -570,8 +591,7 @@ sub _setup_clearer {
     my $name = $self->{name};
 
     my $clearer = $self->{clearer} or return;
-    my $sub = eval qq{ sub { delete \$_[0]{'$name'} } };
-    Moos::_export($metaclass->{package}, $clearer, $sub);
+    $metaclass->add_method($clearer, qq{ delete \$_[0]{'$name'} });
     return;
 }
 
@@ -588,8 +608,7 @@ sub _setup_predicate {
         );
     }
 
-    my $sub = eval qq{ sub { exists \$_[0]{'$name'} } };
-    Moos::_export($metaclass->{package}, $predicate, $sub);
+    $metaclass->add_method($predicate, qq{ exists \$_[0]{'$name'} });
     return;
 }
 
@@ -610,8 +629,12 @@ sub _setup_delegation {
             next if $method =~ $VALID_NAME;
             confess "invalid delegated method name '$method'";
         }
-        my $sub = eval qq{ sub { shift->{'$name'}->$remote(\@_) } };
-        Moos::_export($metaclass->{package}, $local, $sub);
+        if ($self->_is_simple) {
+            $metaclass->add_method($local, qq{ shift->{$name}->$remote(\@_) });
+        }
+        else {
+            $metaclass->add_method($local, qq{ shift->$name\->$remote(\@_) });
+        }
     }
     return;
 }
@@ -714,8 +737,10 @@ Moos - Moo s{imple,peedy,ingle}
 
 Moos completes the M to Moose sequence of Perl OO modules.
 
-This one is pure Perl, no dependencies, single file and Moose compatible (for
-what it does).
+This one is pure Perl, single file and mostly Moose compatible (for what it
+does). Moos has no non-core dependencies, but certain features (roles,
+debugging functions, legacy Perl support) do require additional modules. If
+you steer away from those features, you don't need those additional modules.
 
 =head1 Features
 
@@ -896,6 +921,7 @@ of the following methods:
 C<name>,
 C<attribute_metaclass>,
 C<add_attribute>,
+C<add_method>,
 C<superclasses>,
 C<linearized_isa>,
 C<new_object>,
@@ -945,6 +971,8 @@ into a Moos class, you end up with a strange situation where the accessor
 methods will enforce type constraints (because they were generated by Moose)
 but the constructor will not (because it is inherited from Moos::Object).
 
+See also L<Moos::Role>.
+
 =head2 Method Modifiers
 
 If you need method modifiers, then try L<Class::Method::Modifiers>.
